@@ -1,4 +1,5 @@
 #define FAILUTF8(want)					\
+    parser->bad_beginning = startofutf8string - 1;	\
     parser->bad_type = json_string;			\
     parser->bad_byte = parser->end - 1;			\
     parser->expected = want;				\
@@ -13,7 +14,7 @@
 	ADDBYTE;
 	goto string_start;
     default:
-	FAILUTF8(XNOT_80_BF);
+	FAILUTF8(XBYTES_80_BF);
     }
 
  byte_penultimate_80_bf:
@@ -24,7 +25,7 @@
 	ADDBYTE;
 	goto byte_last_80_bf;
     default:
-	FAILUTF8(XNOT_80_BF);
+	FAILUTF8(XBYTES_80_BF);
     }
 
  byte24_90_bf:
@@ -35,7 +36,7 @@
 	ADDBYTE;
 	goto byte_penultimate_80_bf;
     default:
-	FAILUTF8(XNOT_90_BF);
+	FAILUTF8(XBYTES_90_BF);
     }
 
  byte23_80_9f:
@@ -46,7 +47,7 @@
 	ADDBYTE;
 	goto byte_last_80_bf;
     default:
-	FAILUTF8(XNOT_80_9F);
+	FAILUTF8(XBYTES_80_9F);
     }
 
  byte23_a0_bf:
@@ -57,7 +58,7 @@
 	ADDBYTE;
 	goto byte_last_80_bf;
     default:
-	FAILUTF8(XNOT_A0_BF);
+	FAILUTF8(XBYTES_A0_BF);
     }
 
  byte24_80_bf:
@@ -68,7 +69,7 @@
 	ADDBYTE;
 	goto byte_penultimate_80_bf;
     default:
-	FAILUTF8(XNOT_80_BF);
+	FAILUTF8(XBYTES_80_BF);
     }
 
  byte24_80_8f:
@@ -79,5 +80,5 @@
 	ADDBYTE;
 	goto byte_penultimate_80_bf;
     default:
-	FAILUTF8(XNOT_80_8F);
+	FAILUTF8(XBYTES_80_8F);
     }
